library IEEE;
use IEEE.STD_LOGIC_1164.all;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;

package debug is

procedure dwrite ( message : string );
procedure dwriteVerbose ( message : string );
procedure dwrite ( message : string; slv : std_logic_vector );
procedure dwrite ( message : string; slv1: std_logic_vector; slv2 : std_logic_vector);

end debug;

package body debug is

--  File to log results to
FILE logFile: TEXT OPEN WRITE_MODE IS "results.txt";

-- Set the following to true to generate more messages
constant verbose : boolean := true;

--*******************************************************
-- Write a message to the logfile & transcript
--
--  message => string to write
--
procedure dwrite ( message : string
                   ) is
                   
variable assertMsgBuffer : String(1 to 4096); -- string for assert message
variable writeMsgBuffer : line; -- buffer for write messages

begin
   write( writeMsgBuffer, message );
   assertMsgBuffer(writeMsgBuffer.all'range) := writeMsgBuffer.all;
   writeline(logFile, writeMsgBuffer);
   deallocate(writeMsgBuffer);
   assert (false) report assertMsgBuffer severity note;
end;

--*******************************************************
-- Write a message to the logfile & transcript
-- but only when verbose flag is true
--
--  message => string to write
--
procedure dwriteVerbose ( message : string
                        ) is
begin
   if (verbose) then
      dwrite( message );
   end if;
end;

procedure dwrite ( message : string;
                   slv     : std_logic_vector
                 ) is
                 
variable assertMsgBuffer : String(1 to 4096); -- string for assert message
variable writeMsgBuffer : line; -- buffer for write messages

begin
   write( writeMsgBuffer, message );
   write( writeMsgBuffer, slv );
   assertMsgBuffer(writeMsgBuffer.all'range) := writeMsgBuffer.all;
   writeline(logFile, writeMsgBuffer);
   deallocate(writeMsgBuffer);
   assert (false) report assertMsgBuffer severity note;
end;

procedure dwrite ( message : string;
                   slv1    : std_logic_vector;
                   slv2    : std_logic_vector
                 ) is
                 
variable assertMsgBuffer : String(1 to 4096); -- string for assert message
variable writeMsgBuffer : line; -- buffer for write messages

begin
   write( writeMsgBuffer, message );
   write( writeMsgBuffer, slv1 );
   write( writeMsgBuffer, string'(" : ") );
   write( writeMsgBuffer, slv2 );
   assertMsgBuffer(writeMsgBuffer.all'range) := writeMsgBuffer.all;
   writeline(logFile, writeMsgBuffer);
   deallocate(writeMsgBuffer);
   assert (false) report assertMsgBuffer severity note;
end;

end debug;
