LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY regTestbench IS
END regTestbench;

ARCHITECTURE behavior OF regTestbench IS 

-- input signals
signal reset     : std_logic;
signal clock     : std_logic;
signal address   : std_logic_vector( 4 downto 0);
signal dataIn    : std_logic_vector(31 downto 0);
signal write     : std_logic;

-- output signals
signal dataOut   : std_logic_vector(31 downto 0);

constant clockHigh   : time := 50 ns; 
constant clockLow    : time := 50 ns; 
constant clockPeriod : time := clockHigh + clockLow; 

signal complete : boolean;

begin

   --****************************************************
   -- Clock Generator
   --
   ClockGen:
   process

   begin
      while not complete loop
         clock <= '0';  wait for clockHigh;
         clock <= '1';  wait for clockLow;
      end loop;

      wait; -- stop process looping
   end process ClockGen;

   uut:
  entity work.Registers(BlockRAM) 
--  entity work.Registers(ThreeState)
--entity work.Registers 
   generic map(
      dataWidth    => 32,
      addressWidth => 5
      )
   port map (
      reset     => reset,
      clock     => clock,
      -- Port A
      writeA    => write,
      addressA  => address,
      dataInA   => dataIn,
      -- Port B
      addressB  => address,
      dataOutB  => open,
      -- Port C
      addressC  => address,
      dataOutC  => dataOut
      );

testbench:
process

begin

   reset <= '1';
   wait for 10 ns;
   wait until rising_edge(clock);
   reset <= '0';

--*****************************************
   -- reg(1) <= "11"
   address <= "00001";  dataIn <= x"11111111"; write <= '1';
   wait until rising_edge(clock);
   wait until rising_edge(clock);

   -- reg(2) <= "22"
   address <= "00010";  dataIn <= x"22222222"; write <= '1';
   wait until rising_edge(clock);
   wait until rising_edge(clock);

   -- read mem(1) => "11"
   address <= "00001";  dataIn <= x"00000000"; write <= '0';
   wait until rising_edge(clock);
   wait until rising_edge(clock);
   
   -- read mem(2) => "22"
   address <= "00010";  dataIn <= x"00000000"; write <= '0';
   wait until rising_edge(clock);
   wait until rising_edge(clock);
   
   -- reg(2) <= "2A"
   address <= "00010";  dataIn <= x"2A2A2A2A"; write <= '1';
   wait until rising_edge(clock);

   -- read mem(2) => "2A"
   address <= "00010";  dataIn <= x"00000000"; write <= '0';
   wait until rising_edge(clock);
   wait until rising_edge(clock);
   
   wait for 100 ns;
   --dwrite( string'("Simulation completed.") );

   -- end simulation
   complete <= true;
   wait;

end process testbench;

end architecture behavior;
