--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:   12:13:53 07/17/05
-- Design Name:    
-- Module Name:    Registers
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Registers is
    Generic ( dataWidth    : integer := 32;
              addressWidth : integer := 5
            );
    Port ( reset    : in  std_logic;
           clock    : in  std_logic;
           -- Port A
           writeA   : in  std_logic;
           addressA : in  std_logic_vector(addressWidth-1 downto 0);
           dataInA  : in  std_logic_vector(dataWidth-1    downto 0);
           -- Port B
           addressB : in  std_logic_vector(addressWidth-1 downto 0);
           dataOutB : out std_logic_vector(dataWidth-1    downto 0);
           -- Port C
           addressC : in  std_logic_vector(addressWidth-1 downto 0);
           dataOutC : out std_logic_vector(dataWidth-1    downto 0)
           );
end Registers;

--============================================================================
-- Creates a 3-port (WRR) memory from two 2-port (WR) distributed RAM memories
-- This synthesizes to reasonably efficient hardware
-- Rather brute force using two memories!
--
architecture DistributedRAM of Registers is

component distributedregfile IS
	port (
	clk      : IN  std_logic;

   -- Port A (write)
	we       : IN  std_logic;
	a        : IN  std_logic_VECTOR( 4 downto 0);
	d        : IN  std_logic_VECTOR(31 downto 0);
	spo      : OUT std_logic_VECTOR(31 downto 0);

   -- Port B (read)
	dpra     : IN  std_logic_VECTOR( 4 downto 0);
	dpo      : OUT std_logic_VECTOR(31 downto 0)
   );
end component distributedregfile;

signal write : std_logic;

constant zeroAddress : std_logic_vector(addressWidth-1 downto 0) := (others => '0');

begin

-- one write, one read port
distRegFileAB: distributedregfile
	port map (
	clk   => clock,

   -- Port A (write)
	we    => write,
	a     => addressA,
	d     => dataInA,
   spo   => open,

   -- Port B (read)
	dpra  => addressB,
	dpo   => dataOutB
   );

-- one write, one read port
distRegFileAC: distributedregfile
	port map (
	clk   => clock,

   -- Port A (write)
	we    => write,
	a     => addressA,
	d     => dataInA,
   spo   => open,

   -- Port B (read)
	dpra  => addressC,
	dpo   => dataOutC
   );

  -- don't allow R0 to be changed!
  write <= writeA when (addressA /= zeroAddress) else
           '0';
end architecture DistributedRAM;

--=====================================================================
-- Implementation of 3-port (WRR) memory using Multiplexors
-- This synthesizes to rather inefficient hardware
--
architecture Muxes of Registers is

type data is array (0 to (2**addressWidth)-1) of std_logic_vector(dataWidth-1 downto 0);
signal reg : data;

constant zeroAddress : std_logic_vector(addressWidth-1 downto 0) := (others => '0');

begin

   process (reset, clock)
   begin
      if (reset = '1') then
         reg <= (others => (others => '0'));
      elsif (rising_edge(clock)) then
         -- Don't allow R0 to be changed.
         if (writeA = '1') and (addressA /= zeroAddress) then
            reg(conv_integer(addressA)) <= dataInA;
         end if;
      end if;
   end process;

   dataOutB <= reg(conv_integer(addressB));
   dataOutC <= reg(conv_integer(addressC));

end architecture Muxes;

